@@ -7,13 +7,20 @@ import sys
 import os
 import copy
 import time
+import threading
+import requests
 
 try:
     from . import moves
 except ImportError:
     import moves
 
-VERSION = '0.0.14'
+try:
+    from . import server
+except ImportError:
+    import server
+
+VERSION = '0.0.15'
 
 class Ansi:
     """ The terminal ansi chars """
@@ -477,6 +484,11 @@ OPTIONS
     --player-white=[name]: set name of white player
     --player-black=[name]: set name of black player
     --no-beep: do not play beep sound
+    --online: serve a online game
+    --online --host=[host]: set host of online game
+    --online --port=[port]: set port of online game
+    --online --guess-color=[color]: color of guess player (black or white)
+    --online --name=[name]: set your name white joining to a game
 
 AUTHOR
     This software is created by Parsa Shahmaleki <parsampsh@gmail.com>
@@ -582,6 +594,39 @@ GAME
         $ tchess --replay my-saved-game.file --replay-speed=0.5
 
         (sort of options is not important).
+
+        Online multiplayer
+
+        By default, Tchess runs a offline game for you that you should play on one terminal. Means both of players should use one computer alongside together.
+
+        But you can play with your friend with two computers (on local network).
+
+        Means one player will be Server and othe player will be Guess.
+
+        Game will be handled by Server computer. and guess will be connected to server and play.
+
+        To serve a game, run this command:
+
+        $ tchess --online
+        # OR
+        $ tchess --online --port=<port> --host=<host>
+        # Example
+        $ tchess --online --port=5000 --host=0.0.0.0
+
+        then, the guess player can join the game by running this command:
+
+        $ tchess --connect <host>:<port>
+        # Example
+        $ tchess --connect 192.168.1.2:5000
+
+        Also guess can determine the name:
+
+        $ tchess --connect 192.168.1.2:5000 --name="Guess name"
+
+        Also server player can use more options:
+
+        # set color of guess player (default is black)
+        $ tchess --online --guess-color=white
 '''.strip())
 
 def load_game_from_file(path: str):
@@ -602,6 +647,65 @@ def load_game_from_file(path: str):
     game.current_check = file_game.current_check
     return game
 
+def online_connect(target, options=[], arguments=[]):
+    """ Connects user to a served game """
+    my_name = None
+    for option in options:
+        if option.startswith('--name='):
+            my_name = option.split('=', 1)[1]
+
+    target = 'http://' + target
+    session_id = None
+    my_color = None
+    try:
+        print('Waiting for server confirmation...')
+        connect_args = {}
+        if my_name is not None:
+            connect_args['name'] = my_name
+        res = requests.get(target + '/connect', connect_args)
+        if not res.ok:
+            print('ERROR: invalid response from server: ' + str(res.status_code) + ': ' + res.text, file=sys.stderr)
+            sys.exit(1)
+
+        session_id = res.text.strip()
+        if session_id == '':
+            session_id = None
+
+        if session_id is None:
+            print('ERROR: invalid session id', file=sys.stderr)
+            sys.exit(1)
+
+        # get my color
+        try:
+            my_color = requests.get(target + '/me', {'session': session_id}).text.strip()
+        except:
+            print('ERROR: error while getting guess color', file=sys.stderr)
+            sys.exit(1)
+    except:
+        print('ERROR: cannot make http connection to the target', file=sys.stderr)
+        sys.exit(1)
+
+    while True:
+        print('\033[H', end='')
+        try:
+            render = requests.get(target + '/render', {'session': session_id}).text
+            render = render.split('\n', 1)
+            turn = render[0]
+            render = render[1]
+            print(render)
+            if turn == my_color:
+                command = input(turn + ' Turn >>> ').strip()
+                if command == '':
+                    continue
+                cmd_res = requests.get(target + '/command', {'session': session_id, 'cmd': command})
+                print(cmd_res.text)
+        except KeyboardInterrupt:
+            break
+        except:
+            print('WARNING: unable to connect to server. retrying...', file=sys.stderr)
+            continue
+        time.sleep(0.5)
+
 def run(args=[]):
     """ The main cli entry point """
 
@@ -627,6 +731,15 @@ def run(args=[]):
         options.remove('--no-ansi')
         Ansi.disable()
 
+    # handle `--connect`
+    if '--connect' in options:
+        if len(arguments) <= 0:
+            print('ERROR: <host>:<port> argument is required', file=sys.stderr)
+            sys.exit(1)
+        target = arguments[0]
+        online_connect(target, options, arguments)
+        return
+
     # handle `--replay` option
     is_play = False
     log_counter = 0
@@ -700,6 +813,36 @@ def run(args=[]):
     # last result of runed command
     last_message = ''
 
+    is_online = False
+    game.guess_color = 'black'
+    if '--online' in options:
+        for option in options:
+            if option.startswith('--guess-color='):
+                game.guess_color = option.split('=', 1)[1].lower()
+                if game.guess_color != 'white':
+                    game.guess_color = 'black'
+        is_online = True
+        print('Server is served, waiting for guess...')
+        game.guess_ran = False
+        game.guess_connected = False
+        host = '0.0.0.0'
+        port = 8799
+        for option in options:
+            if option.startswith('--host='):
+                host = option.split('=', 1)[1]
+            elif option.startswith('--port='):
+                try:
+                    port = int(option.split('=', 1)[1])
+                except:
+                    pass
+        server_thread = threading.Thread(target=server.serve, args=[game, host, port])
+        server_thread.daemon = True
+        server_thread.start()
+
+        # wait for connection
+        while not game.guess_connected:
+            pass
+
     while True:
         # render the game board on the terminal
         print('\033[H', end='')
@@ -715,7 +858,7 @@ def run(args=[]):
             # game is finished
             print(Ansi.GREEN + 'Checkmate!' + Ansi.RESET + (' ' * (len(Game.ROW_SEPARATOR)-10)))
             color = Ansi.CYAN if game.winner == 'white' else Ansi.RED
-            print(color + game.winner + Ansi.GREEN + ' won!' + Ansi.RESET)
+            print(color + game.winner + Ansi.GREEN + ' won!' + Ansi.RESET + (' ' * (len(Game.ROW_SEPARATOR)-10)))
             next_step = input('Press enter to continnue or type `back`: ').strip().lower()
             if next_step == 'back':
                 game.is_end = False
@@ -741,7 +884,15 @@ def run(args=[]):
                 sys.exit()
             log_counter += 1
         else:
-            command = input(ansi_color + game.turn + Ansi.RESET + ' Turn >>> ').strip().lower()
+            if is_online and game.turn == game.guess_color:
+                print('Waiting for guess command...')
+                while not game.guess_ran:
+                    pass
+                print(game.guess_ran)
+                game.guess_ran = False
+                continue
+            else:
+                command = input(ansi_color + game.turn + Ansi.RESET + ' Turn >>> ').strip().lower()
 
         game.highlight_cells = []
         game.selected_cell = None
